from setuptools import find_packages, setup

setup(
    name='lipn-ml',
    packages=find_packages(),
    version='0.1.0',
    description='A short description of the project.',
    author='Your name (or your organization/company/team)',
    license='MIT',
)
